#!/usr/bin/env python
#-*- coding:utf-8 -*-

# authors:guanfl
# 2021/6/7


from urllib.parse import quote, unquote, urlencode

import requests
import json
from src.tank.data.tkjson import TkJson

class HttpBase(object):
    def __init__(self,url,
                 params=None,data=None,json=None,
                 json_code=False,text=False,headers=None,proxies=None,
                 cookies=None,verify=True,
                 timeout=None,stream=False,files=None,auth=None,run=None):

        self.verify = verify #SLL验证
        self.url = url #url地址
        self.data = data #数据
        self.params = params #get请求参数
        self.headers = headers # 请求头
        self.proxies = proxies # ip地址代理
        self.cookies = cookies
        self.timeout = timeout #响应等待时间
        self.stream = stream #获取原始套接字响应 需要True
        self.files = files #文件类型
        self.auth = auth #登录验证 需要传递元组 用户名和密码

        self._tkjson = TkJson()

        # if run is None:
        #
        #     self.run(params=self.params,data=data,json=json,
        #              json_code=json_code,text=text,headers=headers,cookies=cookies)
        #

    # def run(self,params=None,data=None,json=None,
    #              json_code=False,text=False,headers=None,cookies=None):
    #     return self.http_send(params=params,
    #                               data=data,json=json,json_code=json_code,text=text)


    #get 请求
    def http_send(self,method,params=None,data=None,json=None,json_code=None,text=True):
        if method == "get":
            return self.get(url=self.url,params=self.params,headers=self.headers,
                            cookies=self.cookies,verify=self.verify,timeout=self.timeout,
                            stream=self.stream)

        elif method == "post" and self.url:
            return self.post(url=self.url,data=self.data, json=json,headers=self.headers,
                             proxies=self.proxies,cookies=self.cookies,verify=self.verify,
                             timeout=self.timeout,stream=self.stream,
                             files=self.files,auth=self.auth)

    #url编码
    def quote(self,data):
        return quote(data)

    def json_format(self):
        return self._tkjson.json_cn_format(self.json())

    #解码
    def unquote(self,encode_data):
        return  unquote(encode_data)

    #url解码
    def url_uote(self,encode_data):
        return  requests.utils.unquote(encode_data)

    #url编码
    def url_quote(self,encode_data):
        return  requests.utils.quote(encode_data)

    #参数编码
    def query_encode(self,my_query):
        return urlencode(my_query)

    #原始响应内容
    def r(self):
        return self.run().raw

    #读取原始内容
    def read(self,index=None):
        return self.r().read(index)

    #保存文件 chunk_size字节流
    def iter(self,filename,chunk_size=100):
        with open(filename, 'wb') as fd:
            for chunk in self.run().iter_content(chunk_size):
                fd.write(chunk)

    #获取源码
    def text(self):
        return self.run().text

    def json(self):
        return self.run().json()

    #获取二进制数据
    def content(self):
        return self.run().content

    def dict(self):
        return json.loads(self.decode())

    #二进制数据解码
    def decode(self):
        return self.content().decode()


    #获取编码方式
    def encoding(self):
        return self.run().encoding

    # self.request.encoding = "utf-8" 改变编码格式

    #请求状态码
    def request_status_code(self):
        return self.run().status_code

    #请求url
    def request_url(self):
        return  self.run().request.url

    # #请求cookies
    # def request_cookies(self):
    #     return  self.run().request.cookies

    #请求头
    def request_headers(self):
        return  self.run().request.headers

    # #请求方式
    # def get_request(self):
    #     md = self.run().request
    #     jk = type(md)
    #     if md == jk:
    #         return "GET"

    #保存cookies 至session中
    # def session(self):
    #     return requests.session().get()

    #响应状态码
    def response_status_code(self):
        return self.run().content

    #响应头
    def response_headers(self):
        return self.run().headers

    #响应cookies
    def response_cookies(self):
        return self.run().cookies

    #设置指定响应cookies转换字典
    def cookies_dict(self,cookies):
        # self.response_cookies()
        return requests.utils.dict_from_cookiejar(cookies)


    #获取当前响应cookies转换字典
    def get_cookies_dict(self,cookies):
        return requests.utils.dict_from_cookiejar(self.response_cookies())



    #下载二进制文件
    def bin_file(self,fileName):
        with open(fileName,"wb") as f:
            f.write(self.content())


    def get(self,url,params=None,headers=None,cookies=None,verify=None,timeout=None,stream=None):
        return requests.get(url=url,params=params,headers=headers,cookies=cookies,
                            verify=verify,timeout=timeout,stream=stream)


    def post(self,url,data=None,json=None,headers=None,proxies=None,cookies=None,
             verify=None,timeout=None,stream=None,files=None,auth=None):
        return  requests.post(url=url,data=data,json=json,headers=headers,proxies=proxies,
                              cookies=cookies,verify=verify,timeout=timeout,
                              stream=stream,files=files,auth=auth)


    #断言 等值状态码
    def assert_status(self,status):
        try:
            assert self.request_status_code() == status
            return True
        except:
            return False

def file_format(file):
    file = {'file': open(file, 'rb')}
    return file




# data 类型
#非结构化 html 数据
#处理方法 ：xpath 正则表达式

#结构化数据 json xml 等
#处理方法 转化为python数据类型
