import datetime
import logging.config
import os

from tank.default.conf import settings


base = getattr(settings,'BASE_DIR')
path = os.path.join(base,'report/logs').replace("\\",'/')
# 定义三种日志输出格式 开始
standard_format = '%(levelname)s  %(asctime)s  %(threadName)s:%(thread)d_task_id:%(name)s_%(filename)s:%(lineno)d' \
                  ' msg:%(message)s' #其中name为getlogger指定的名字

simple_format = '%(levelname)s  %(asctime)s %(filename)s:%(lineno)d' \
                  ' msg:%(message)s'

id_simple_format = '[%(levelname)s][%(asctime)s] %(message)s'
error = os.path.join(path,datetime.datetime.now().strftime("%Y-%m-%d") + "_error"+".log").replace('\\','/')
info = os.path.join(path,datetime.datetime.now().strftime("%Y-%m-%d") + ".log").replace('\\','/')


import threading




# log配置字典
LOGGING_CONFIG = {
    'version': 1,
    # 禁用已经存在的logger实例
    'disable_existing_loggers': False,
    # 定义日志 格式化的 工具
    'formatters': {
        'standard': {
            'format': standard_format
        },
        'simple': {
            'format': simple_format
        },
        'id_simple': {
            'format': id_simple_format
        },
    },
    # 过滤器
    'filters': {},
    'handlers': {
        #打印到终端的日志
        'stream': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',  # 打印到屏幕
            'formatter': 'simple'
        },
        #打印到文件的日志,收集info及以上的日志
        'file': {
            'level': 'DEBUG',
            'class': 'logging.handlers.RotatingFileHandler',  # 保存到文件
            'formatter': 'standard',
            'filename': info, # 日志文件路径
            'maxBytes': 1024*1024*5,  # 日志大小 5M
            'backupCount': 5,
            'encoding': 'utf-8',  # 日志文件的编码
        },
        #打印到文件的日志,收集error及以上的日志
        'debug': {
                    'level': 'ERROR',
                    'class': 'logging.handlers.RotatingFileHandler',  # 保存到文件
                    'formatter': 'id_simple',
                    'filename': error,
                    'maxBytes': 1024*1024*5,  # 数字时为多少时间进行分割文件 30s秒
                    'backupCount': 5,
                    'encoding': 'utf-8',
                },
    },
    # logger实例
    'loggers': {
        # 默认的logger应用如下配置
        'api': {
            'handlers': ['stream', 'file','debug'],  # 这里把上面定义的两个handler都加上，即log数据既写入文件又打印到屏幕
            'level': 'DEBUG',
            'propagate': True,  # 向上（更高level的logger）传递
        },
        # logging.getLogger(__name__)拿到的logger配置
    },
}


class TankLog:
    def __init__(self):
        logging.config.dictConfig(LOGGING_CONFIG)
        self.logger = logging.getLogger("api")
        self.logger_conf = getattr(settings, 'LOGGING_STATUS')

    def info(self,msg):
        if self.logger_conf is True:
            self.logger.info(msg=msg)

    def debug(self,msg):
        self.logger.debug(msg=msg)

    def error(self,msg):
        self.logger.error(msg=msg)


# class PyLog4j:
#     def __init__(self,conf,logger):
#         """
#         :param conf:
#         :param logger:
#         """
#         logging.config.dictConfig(conf)
#         self.logger = logging.getLogger(logger)
#
#     def info(self,msg):
#         self.logger.info(msg=msg)
#
#     def debug(self,msg):
#         self.logger.debug(msg=msg)
#
#     def error(self,msg):
#         self.logger.error(msg=msg)


if __name__ == '__main__':
    pass
