#!/usr/bin/env python
#-*- coding:utf-8 -*-

# authors:guanfl
# 2021/4/27


"""统一执行测试用例，并生成测试报告"""



import os
import time

from tank.default.conf import settings

# from core.dingdingWebhook import DingdingWebhook

# from core.oss import Upload
from tank.core.common import is_dir
from tank.core.logs import TankLog
from tank.tkunit.report.TkRunner import HTMLTestRunner
from tank import tkunit
from tank.core.tankos import ZipFile
from tank.core.tkemail import UnittestMail

TEST_REPORT = {
    'context_processors':
        {
            'zip':'',
            'html':'',
            'img':'',
        }
}



log4j = TankLog()

# guanfl
# v1.0.1
# 20210602
# :TODO :完成第一版 待单元测试
class TkMainTestSuite(object):
    def __init__(self,report_filename=None):
        if report_filename is None:self.report_filename = "自动化测试报告"
        else:self.report_filename = report_filename

    #     服务启动
        if getattr(settings, "API_TEST_TEMPLATE") is True:
            pass

    def test_suite_base(self,test_title_name=None,description=None):
        """
        基本的测试集
        :param test_title_name:
        :param description:
        :return:
        """
        get_case = getattr(settings, "CASE_SUITE_PATH")

        # 定义测试执行器
        suite = tkunit.defaultTestLoader.discover(start_dir=get_case, pattern=getattr(settings, "RUN_PATTERN_CASE"))

        # #测试报告保存的路径
        now = time.strftime('%Y-%m-%d', time.localtime())
        get_html = getattr(settings, "TEST_REPORT")

        # 如果有配置html报告路径则进行使用配置里的数据、没有将进行使用tank默认的存放路径
        if not get_html["context_processors"]:
            get_base= getattr(settings, "BASE_DIR")
            path = os.path.join(get_base,'report/html',now).replace('\\','/')
        else:
            path = os.path.join(get_html["context_processors"]["html"],now).replace('\\','/')

        # 判断当 report_base 目录是否已存在、如果不存在就进行创建新的目录
        try:
            if is_dir(path) is False:
                pass
            else:
                log4j.info(msg=f"正在创建{path}目录")
                os.mkdir(path)
        except Exception as e:
            log4j.error(msg=e)
        #
        #
        joinPath = os.path.join(path,'%s_%s.html'%(self.report_filename,now)).replace("\\","/")
        #
        fp = open(joinPath, 'wb')

        get_retry = getattr(settings, "TEST_CASE_ERROR_RETRY")
        # 测试过程中的数据流写入到打开的报告中
        runner = HTMLTestRunner(
        stream=fp, retry=get_retry, title=test_title_name,
            description=description, save_last_try=True,verbosity=3
        )
        #
        text_content = runner.run(suite)
        return  text_content

    def test_suite_report_zip(self,test_title_name=None,description=None):
        """
        可以将测试报告进行打包
        :param test_title_name:
        :param description:
        :return:
        """
        get_case = getattr(settings, "CASE_SUITE_PATH")

        # 定义测试执行器
        suite = tkunit.defaultTestLoader.discover(start_dir=get_case, pattern=getattr(settings, "RUN_PATTERN_CASE"))

        # #测试报告保存的路径
        now = time.strftime('%Y-%m-%d', time.localtime())
        get_html = getattr(settings, "TEST_REPORT")

        # 如果有配置html报告路径则进行使用配置里的数据、没有将进行使用tank默认的存放路径
        if not get_html["context_processors"]:
            get_base = getattr(settings, "BASE_DIR")
            path = os.path.join(get_base, 'report/html', now).replace('\\', '/')
        else:
            path = os.path.join(get_html["context_processors"]["html"], now).replace('\\', '/')

        # 判断当 report_base 目录是否已存在、如果不存在就进行创建新的目录
        try:
            if is_dir(path) is False:
                pass
            else:
                log4j.info(msg=f"正在创建{path}目录")
                os.mkdir(path)
        except Exception as e:
            log4j.error(msg=e)
        #
        #
        joinPath = os.path.join(path, '%s_%s.html' % (self.report_filename,now)).replace("\\", "/")
        #
        fp = open(joinPath, 'wb')

        get_retry = getattr(settings, "TEST_CASE_ERROR_RETRY")
        # 测试过程中的数据流写入到打开的报告中
        runner = HTMLTestRunner(
            stream=fp, retry=get_retry, title=test_title_name,
            description=description, save_last_try=True, verbosity=3
        )
        #
        text_content = runner.run(suite)

        # 进行对测试结果进行打包成一个zip压缩包
        log4j.info(msg="正在准备打包测试报告为zip,本次打包的目录路径为：{}".format(path))
        # 生成手写识别打包成zip文件的目录路径

        if not get_html["context_processors"]:
            get_base = getattr(settings, "BASE_DIR")
            zip_file_report_path = os.path.join(get_base, 'report/zip/%s_%s.zip' % (self.report_filename, now)).replace("\\", "/")
        else:
            zip_file_report_path = os.path.join(get_html["context_processors"]["zip"],
                                           '%s_%s.zip' % (self.report_filename, now)).replace("\\", "/")

        zip_path_save = ZipFile()
        path = zip_path_save.zip_file(path=path,zipfile_name=zip_file_report_path)
        log4j.info(msg="已对目录路径为：{},下的文件进行打包成zip文件,存放路径为：{}".format(path,zip_file_report_path))

        return text_content

    def test_suite_send_email(self,context,test_title_name=None,description=None,email_title=None,filename=None):
        """
        自动邮件
        :param test_title_name:
        :param description:
        :return:
        """
        get_case = getattr(settings, "CASE_SUITE_PATH")

        # 定义测试执行器
        suite = tkunit.defaultTestLoader.discover(start_dir=get_case, pattern=getattr(settings, "RUN_PATTERN_CASE"))

        # #测试报告保存的路径
        now = time.strftime('%Y-%m-%d', time.localtime())
        get_html = getattr(settings, "TEST_REPORT")

        # 如果有配置html报告路径则进行使用配置里的数据、没有将进行使用tank默认的存放路径
        if not get_html["context_processors"]:
            get_base = getattr(settings, "BASE_DIR")
            path = os.path.join(get_base, 'report/html', now).replace('\\', '/')
        else:
            path = os.path.join(get_html["context_processors"]["html"], now).replace('\\', '/')

        # 判断当 report_base 目录是否已存在、如果不存在就进行创建新的目录
        try:
            if is_dir(path) is False:
                pass
            else:
                log4j.info(msg=f"正在创建{path}目录")
                os.mkdir(path)
        except Exception as e:
            log4j.error(msg=e)
        #
        #
        joinPath = os.path.join(path, '%s_%s.html' % (self.report_filename,now)).replace("\\", "/")
        #
        fp = open(joinPath, 'wb')

        get_retry = getattr(settings, "TEST_CASE_ERROR_RETRY")
        # 测试过程中的数据流写入到打开的报告中
        runner = HTMLTestRunner(
            stream=fp, retry=get_retry, title=test_title_name,
            description=description, save_last_try=True, verbosity=3
        )
        #
        text_content = runner.run(suite)

        # 进行对测试结果进行打包成一个zip压缩包
        log4j.info(msg="正在准备打包测试报告为zip,本次打包的目录路径为：{}".format(path))
        # 生成手写识别打包成zip文件的目录路径

        if not get_html["context_processors"]:
            get_base = getattr(settings, "BASE_DIR")
            zip_file_report_path = os.path.join(get_base, 'report/zip/%s_%s.zip' % (self.report_filename, now)).replace("\\", "/")
        else:
            zip_file_report_path = os.path.join(get_html["context_processors"]["zip"],
                                           '%s_%s.zip' % (self.report_filename, now)).replace("\\", "/")

        zip_path_save = ZipFile()
        path = zip_path_save.zip_file(path=path,zipfile_name=zip_file_report_path)
        log4j.info(msg="已对目录路径为：{},下的文件进行打包成zip文件,存放路径为：{}".format(path,zip_file_report_path))

        time.sleep(2)
        if email_title is None:title = "自动化测试报告"
        else:title = email_title

        # 自动邮件发送
        get_bug =  getattr(settings, "DEBUG")
        get_email_list =  getattr(settings, "EMAIL_LISTS")
        try:
            if filename is None:fileName = "result.zip"
            else:fileName = filename

            get_email_sender =  getattr(settings, "EMAIL_SENDER")

            if get_bug is False:
                if not get_email_sender["context_processors"]:
                    log4j.debug(msg="你还没有进行配置EMAIL_SENDER,请配置 EMAIL_SENDER")
                else:
                    UnittestMail(sender=get_email_sender["context_processors"]["username"], password=get_email_sender["context_processors"]["password"],
                                 toaddrs=get_email_list,
                                 title=title, htmlPATH=zip_file_report_path, FILEname=fileName,
                                 content=context)
        except Exception as e:
            log4j.error(e)

        return  text_content

        #
        #
        #         for res in text_content[1][8]:
        #             t = res[0].split(":")
        #             # des  = des + "".format(t[1],res[1],res[2],res[3],res[4])
        #             context = context + "\n### 模块：{}  \n描述：{}         \n总计:{}\n成功:{}\n失败:{}\n错误:{}\n".format(t[0], t[1],
        #                                                                                                      res[1],
        #                                                                                                      res[2], res[3],
        #                                                                                                      res[4])
        #         print(des)
        #
        #         content = "# {}\n## ------基本信息-------\n### 本轮测试总用例数：{} 条\n### 测试执行通过用例数: {} 条\n### 测试执行失败用例数 {} 条\n" \
        #                   "### 测试执行错误用例数: {} 条\n ### 本次测试执行通过率：{} \n### 开始时间：{}\n" \
        #                   "### 运行时间：{}\n## ------测试详情-------\n{}" \
        #                   "".format(title, text_content[1][0], text_content[1][1], text_content[1][2],
        #                             text_content[1][3], text_content[1][4], text_content[1][5], text_content[1][7], context,
        #                             )
        #         print(content)
        #
        #         dd = DingdingWebhook()
        #         print(dd.send_markdown(content=content, title="测试测试"))
        #
        #         # 将测试的结果发送到邮件中
        #         result_path = os.path.join(TEST_REPORT_IMG_CASE_FILE_PATH, "character").replace("\\", "/")
        #         case = os.path.join(TEST_REPORT_IMG_CASE_FILE_PATH, "%s_case_result.zip" % now).replace("\\", "/")
        #
        #         save_path = zip_file(path=result_path, zipfile_name=case)
        #
        #         UnittestMail(sender="gfl13453001@163.com", password="gfl802300",
        #                      toaddrs=EMAIL_LISTS,
        #                      title="这是测试用例识别出现差异的文本结果", htmlPATH=save_path, FILEname="result.zip", content="这是测试用例识别出现差异的文本结果")
        #
        #
        #
        #
        #
        #     elif DEBUG is True and AUTO_SEND_EMAIL is True:
        #         context = ""
        #         des = ""
        #         for res in text_content[1][8]:
        #             t = res[0].split(":")
        #             # des  = des + "".format(t[1],res[1],res[2],res[3],res[4])
        #             context = context + "\n### 模块：{}  \n描述：{}         \n总计:{}\n成功:{}\n失败:{}\n错误:{}\n".format(t[0], t[1],
        #                                                                                                      res[1],
        #                                                                                                      res[2], res[3],
        #                                                                                                      res[4])
        #
        #
        #         UnittestMail(sender="gfl13453001@163.com", password="gfl802300",
        #                      toaddrs=EMAIL_LISTS,
        #                      title=title, htmlPATH=path, FILEname="测试报告.zip", content="这是本次自动化测试执行的测试结果")
        #
        #
        #         # 将测试的结果发送到邮件中
        #         result_path = os.path.join(TEST_REPORT_IMG_CASE_FILE_PATH, "character").replace("\\", "/")
        #         case = os.path.join(TEST_REPORT_IMG_CASE_FILE_PATH, "%s_case_result.zip" % now).replace("\\", "/")
        #
        #         save_path = zip_file(path=result_path, zipfile_name=case)
        #
        #         UnittestMail(sender="gfl13453001@163.com", password="gfl802300",
        #                      toaddrs=EMAIL_LISTS,
        #                      title="这是测试用例识别出现差异的文本结果", htmlPATH=save_path, FILEname="result.zip", content="这是本次自动化测试执行的测试结果")
        #
        #
        #         content = "# {}\n## ------基本信息-------\n### 本轮测试总用例数：{} 条\n### 测试执行通过用例数: {} 条\n### 测试执行失败用例数 {} 条\n" \
        #                   "### 测试执行错误用例数: {} 条\n ### 本次测试执行通过率：{} \n### 开始时间：{}\n" \
        #                   "### 运行时间：{}\n## ------测试详情-------\n{}" \
        #                   "".format(title, text_content[1][0], text_content[1][1], text_content[1][2],
        #                             text_content[1][3], text_content[1][4], text_content[1][5], text_content[1][7], context,
        #                             )
        #         print(content)
        #
        #         dd = DingdingWebhook()
        #         print(dd.send_markdown(content=content, title="测试测试"))


        # except Exception as e:
        #     log4j.error(msg=e)








if __name__ == '__main__':

    case_index = TestSuite()
    print(case_index.tk_test_suite())

